import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Album } from './album.entity';
import { CreateAlbumDto } from './dto/create-album.dto';
import { UpdateAlbumDto } from './dto/update-album.dto';

@Injectable()
export class AlbumService {
  constructor(
    @InjectRepository(Album)
    private albumRepository: Repository<Album>
  ) {}

  findAll(): Promise<Album[]> {
    return this.albumRepository.find({
      order: {
        artist: "ASC"
      }
    });
  }

  findOneById(id: number): Promise<Album | null> {
    return this.albumRepository.findOne({
      where: {
        id: id
      },
      relations: {
        songs: true
      }
    });
  }

  async create(createAlbumDto: CreateAlbumDto): Promise<number> {
    const album = this.albumRepository.create({
      title: createAlbumDto.title,
      artist: createAlbumDto.artist,
      genre: createAlbumDto.genre
    });
    const savedAlbum = await this.albumRepository.save(album);
    return savedAlbum.id;
  }

  async update(id: number, updateAlbumDto: UpdateAlbumDto): Promise<string> {
    if (id === updateAlbumDto.id) {
      const album = this.albumRepository.findOneBy({ id: updateAlbumDto.id });
      if (!album)
        return "Album not found";

      await this.albumRepository.update({ id: updateAlbumDto.id }, {
        title: updateAlbumDto.title,
        artist: updateAlbumDto.artist,
        genre: updateAlbumDto.genre
      });
      return "Album updated successfully";
    } else {
      return "Album ID does not match posted data";
    }
  }

  async remove(id: number): Promise<void> {
    await this.albumRepository.delete(id);
  }
}
